

import pyglet
import pyglet.window.key as key


class Cosmonaut (pyglet.sprite.Sprite):

    def __init__(self, **kwargs):
        pyglet.sprite.Sprite.__init__(self, pyglet.image.create(1, 1), **kwargs)
        self.scale = 8

        self.positionX, self.positionY = 0, 0

        self.moving = False
        self.direction = "down"
        
        self.stepCounter = 0
        self.stepDirection = "down"

        self.keyPresses = []
        self.pressedCounter = 0
        
        spriteSheet = pyglet.resource.image("cosmonautSS.png")
        spriteGrid = pyglet.image.ImageGrid(spriteSheet, 4, 4)
        ft = 0.15
        self.animations = { "walkingFront": pyglet.image.Animation.from_image_sequence(spriteGrid[:4], ft),
                            "walkingBack": pyglet.image.Animation.from_image_sequence(spriteGrid[4:8], ft),
                            "walkingLeft": pyglet.image.Animation.from_image_sequence(spriteGrid[8:12], ft),
                            "walkingRight": pyglet.image.Animation.from_image_sequence(spriteGrid[12:16], ft),
                            "standingFront": pyglet.image.Animation.from_image_sequence(spriteGrid[0:1], ft),
                            "standingBack": pyglet.image.Animation.from_image_sequence(spriteGrid[4:5], ft),
                            "standingLeft": pyglet.image.Animation.from_image_sequence(spriteGrid[8:9], ft),
                            "standingRight": pyglet.image.Animation.from_image_sequence(spriteGrid[12:13], ft),
                          }
        self.animation = ""
        self.goto_animation("walkingFront")


    def update(self, level, dt):
        self.handle_input(level)
        self.update_position()
        self.update_animation()


    def handle_input(self, level):
        for pressedKey in (key.LEFT, key.RIGHT, key.UP, key.DOWN):
            # add keys
            if level.keys[pressedKey] and not pressedKey in self.keyPresses:
                self.keyPresses.append(pressedKey)
            
            # remove keys
            if not level.keys[pressedKey] and pressedKey in self.keyPresses:
                # reset pressedCounter
                if self.keyPresses[-1] == pressedKey:
                    self.pressedCounter = 0
                self.keyPresses.remove(pressedKey)
            
            # increment pressedCounter
            if level.keys[pressedKey] and pressedKey == self.keyPresses[-1]:
                self.pressedCounter += 1

        if self.pressedCounter > 2:
            self.moving = True
        
        if self.keyPresses == []:
            self.moving = False
        else:
            lk = self.keyPresses[-1]
            if lk == key.LEFT:  self.direction = "left"
            if lk == key.RIGHT: self.direction = "right"
            if lk == key.UP:    self.direction = "up"
            if lk == key.DOWN:  self.direction = "down"


    def update_position(self):
        if self.stepCounter:
            if self.stepDirection == "right": self.positionX += 1
            if self.stepDirection == "left":  self.positionX -= 1
            if self.stepDirection == "up":    self.positionY += 1
            if self.stepDirection == "down":  self.positionY -= 1
            self.stepCounter -= 1
        else:
            if self.moving:
                self.stepCounter = 8
                self.stepDirection = self.direction


    def update_animation(self):
        directions = {"left": "Left", "right": "Right", "up": "Back", "down": "Front"}
        self.goto_animation( ("walking" if self.moving or self.stepCounter else "standing") +\
                                        directions[self.stepDirection if self.stepCounter else self.direction] )


    def goto_animation(self, animation):
        if animation != self.animation:
            self.animation = animation
            self.image = self.animations[animation]